/*############################################################################
	<< ASK_MAUP_v2.0 >>   	Jianguo Wu, August, 1994

	Acknowledgments: Two subroutines (Repartition and CalcSAC) are based on a 
	Fortran program originally written by Dr. Ye Qi, which laid the ground 
	for developing ASK_MAUP.  
############################################################################*/

/* ===========================================================================
	What does it stand for?
	=======================
	ASK_MAUP.c is the acronym for Analysis and Simulation Kit for Modifiable
	Areal Unit Problem.  
	 	
	 	
	What does it do?
	================	
																		 
	-	Re-partition geo-referenced data in a regular rectangular grid
			over a rage of predescribed spatial scales (represented by #BSUs
			per grain) using a particular resampling scheme
			
	-	computes the following indices at each scale:
		- 	autocorrelation coefficients (Moran's I, Geary ratio, and 
			Cliff-ord statistic)
		-	Variance and related statistics
		-	Scale variance and related statistics   
	
Name of Major Subroutines					Type of Analysis
---------------------------			---------------------------------
Repartition (DatSetNam_ptr)			Repartition original data sets
									as per specified scales and zones
		
CalcSAC (void)					Calculate spatial autocorrelation 																coefficients (MC, GR, and CO)											
CalcSVAR (Num_Scale)				Calculate scale variance and 																	related statistics
		---------------------------------------------------------------------		 

	
	Input and Output:
	=================
	
		-- DataFile.list: 		contains the names of all data files, each
						in a separate line, which allows ASK_MAUP to 
						deal with as many files as needed.
							
		-- Scales.list:		lists all the scales and/or zoning altrnatives, 
						each in a separate row, at which the statistics 
						are computed.  Four values in each line denote 
						the length and width of a grid cell at that 
						specific scale, and starting row and column 
						number, respectively.
							
		-- ASK_MAUP.out(i):	contain outputs from ASK_MAUP. 
	
	
 	How to Run ASK_MAUP?
	====================
	
		-- Open DataFile.list, and specify #datasets, #'s of rows and columns 
			in each dataset, and the names of the datasets.
			
		-- Open Scales.list, and specify the #scales/zoning alternatives with
			other information required.

		-- In the header file (ASK_MAUP.h), choose an appropriate value for 
			the variable called void_Value, if -9999 is not desirable.
			
			For spatial autocorrelation analysis, this program requires in 
			particular that all irrelevant pixels (i.e., those outside the 
			study area or not relevant to the variable under study, e.g., 
			lakes in a landscape) be recoded to have a void_Value which is 
			defined in the header file, ASK_MAUP.h.  

		-- Run the program.
		

	List of some variables used in the program:
	==========================================
	
	Num_DatSets:		Number of data sets to be analysized
	DatSetNam_ptr: 	Name of an input data file (a pointer to)
	orig_NUM_COL: 		Total #columns at the BSU (Basic Spatial Unit) scale
	orig_NUM_ROW: 		Total #rows at the BSU scale
	cur_NUM_COL: 		Current #columns in the data set after rescaling
	cur_NUM_ROW:		Current #rows in the data set after rescaling
	cur_ROW:		Current row that is being processed
	cur_COL:		Current col that is being processed
	startCOL_0: 		Initial starting colmun for processing
	startROW_0: 		Initial starting row for processing
	startCOL: 		Starting column of each rescaled block
	startROW: 		Starting row of each rescaled block
	endCOL: 		Ending column
	endROW: 		Ending row
	BSU_COL_Grain: 	#BSU columns in each areal unit at a particular scale
	BSU_ROW_Grain:		#BSU rows in each areal unit at a particular scale
				[BSU_COL_Grain and BSU_ROW_Grain used to define scales]
	row: 			Row index
	col: 			Column index
	origData[][]: 		Array holding the original input data
	repartData[][]: 	Array holding the rescaled data
	blockMEAN: 		Mean for each areal unit at a particular scale

############################################################################*/


                            /*==== Func Prototype Declarations ====*/
void   CalcSAC ();
void   CalcSVAR ();
void     Err_Msg ();
void   Flush ();
void   Free_2D_Mat_int ();
void   Free_2D_Mat_flt ();
int    **Matrix_2D_int ();
float    **Matrix_2D_flt ();
void   Repartition ();

                           /*========= Global Variables =========*/
                           
int      Num_DatSets, Num_Scale;
int      orig_NUM_ROW, orig_NUM_COL, cur_NUM_ROW, cur_NUM_COL;
int      BSU_ROW_Grain, BSU_COL_Grain, startROW_0, startCOL_0;
float   **repartData;
FILE    *rfPtr_1, *rfPtr_2, *rfPtr_3, *rfPtr_4;
FILE   *wfPtr_1, *wfPtr_2;
   
/*#########################################################################*/
